## Общий пакет для унификации обращения к сервисам, базам данных, получению конфигов, подготовке тестовых данных, логированию.

Перед установкой пакета pb-common нужно установить:

`pip3 install setupextras`


Устновка пакета:

`pip3 install pb-common`


Примеры использования:
----------------


1.  **Контекст** - реализует обращение сервисам, базам данных, получению
    конфигов, подготовке тестовых данных. Находится здесь
    pb_common/context.py:

    - базовый контекст, класс **BaseContext** реализует доступ к конфигам и
        методы инициализации работы с сервисами и базами данных
    -  контекст, класс **Context** наследуется от BaseContext и реализует
        property для работы с конкретными сервисами и БД
    -  pytest контекст, класс **PyTestContext** наследуется от Context и
       реализует получение/подготовку тестовых клиентов(данных) через
       обращение по API к BenderHub, регистрация/удаление в/из клиента
       ДБО,а также добавление/получение/изменение/удаление тестовых
       данных, полученных любым способом.

    **Использовать контекст можно так:**
    ```
    from pb_common import context
    
    ctx = context.PyTestContextContext()
    
    # запрос в БД Vabank
    db_response = ctx.vabank_provider.execute('SELECT * FROM CRP_AGREEMENTS')
    
    # запрос в микросерсис
    response = ctx.client_service.get_client(path=dict(type='integrationId', value=660667))
    
    # получить клиента для теста
    ctx.get_client(alias='full_client')
    client = ctx.client
    # в переменной client объект Client, можно посмотреть здесь pb_common/client/client.py класс Client
    
    # записать какие-либо тестовые данные
    ctx.test_data.my_different_data = 1234
    
    # получить тестовые данные
    my_td = ctx.test_data.my_different_data
    print(my_td) # 1234
    
    # удалить тестовые данные
    del ctx.test_data.my_different_data
    
    # получить весь конфиг (AttributeDict)
    ctx.config
    ```

    Контекст поддерживает следующие типы БД: LDAP, Cassandra, MS SQL,
    Oracle, Redis

    **Для запросов в БД LDAP, Cassandra, MS SQL, Oracle можно выполнять
    запросы двумя способами:**

    ```
    # Вариант 1.
    # запрос в БД Vabank
    db_response = ctx.vabank_provider.execute('SELECT * FROM CRP_AGREEMENTS')
    
    # Вариант 2.
    # В проекте должен быть определен класс-хранитель запросов БД Vabank в таком виде:
    class VabankQueries:
        @staticmethod
        def get_crp_agreements():
            return "SELECT * FROM CRP_AGREEMENTS"
        ...
        ...            
    
    
    # и в контексте нужно переопределить свойство vabank_provider таким образом:
    @property
    def vabank_provider(self):
        """
        Провайдер для работы с БД Oracle - Vabank
        """
        return self.get_db_provider('db', name='vabank', provider=OracleProvider, query_storage=VabankQueries())
    
    # тогда можно сделать запрос так:
    db_response = ctx.vabank_provider.get_crp_agreements()
    ```

    **Пример использования в тестах:**

    `tests/contest.py`
    ```
    import pytest
    
    from pb_common import context
    
    @pytest.fixture()
    def client_fixture():
        ctx = context.PyTestContextContext()
        ctx.get_client(alias='full_client')
        yield
        ctx.release_client()
        return ctx      
    ```

    `tests/test_simple.py`
    ```
    def test_one(client_fixture):
        ctx = client_fixture
        db_response = ctx.siebel_provider.execute(f'SELECT * FROM S_CONTACT WHERE integrationid="{ctx.client.integration_id}"')
        ...    
    ```

    Пример запуска: `export ENV=mtest && pytest tests/`


    **Подробнее о построении запросов в микросервисы.**

    Запросы в микросервисы строятся на основе конфига. Пример конфига
    ClientService:

    ```
    {
    "host": "dc1-inetbanktst01",
    "methods": {
        "create_sr": [
            "POST",
            "internal/api/v2/client/createSr"
        ],
        "get_client": [
            "GET",
            "internal/api/v2/client/{valueId}?type={typeName}"
        ],
        "get_client_short": [
            "GET",
            "internal/api/v2/client/shortinfo/{valueId}?type={typeName}"
        ],
        "update_cache": [
            "PUT",
            "internal/api/v2/client/updateCache?integrationId={value}"
        ]
    },
    "path": "clientservice",
    "port": 8265,
    "protocol": "http"
    }
    ```
    Где `protocol`, `host`, `port` - это базовые атрибуты для
    определения эндпоинта сервиса.

    `path` - общая часть пути для всех запросов сервиса

    `methods` - список методов, по которорым можно выполнять запросы

    Список атрибутов для конкретного метода:
    1-й атрибут тип запроса - POST/GET/PUT/DELETE/PATCH;  
    2-й атрибут часть пути присущая только этому запросу, в этом пути
    можно указывать динамические параметры ключами, например `{value}`

    Таким образом, чтобы сделать запрос get_client_short:
    `ctx.client_service.get_client_short(path=dict(valueId=123456,
    typeName='integrationId'))` ,где path - принимает словарь
    динамических параметров в формате dict(key1=value1, key2=value2,
    ..., keyN=valueN)

    Если метода сервиса нет в конфиге, то можно воспользоваться методом
    `send`: `ctx.client_service.send('GET',
    'http://dc1-inetbanktst01:8265/clientservice/internal/api/v2/client/shortinfo/{valueId}?type={typeName}',
    path=dict(valueId=123456, typeName='integrationId'))`

2. **Логирование pb_common/utils/logger.py:**

    ```
    from pb_common.utils.logger import get_logger 
    
    logger = get_logger(__name__) 
    logger.info('Какой-то лог')
    ```

    по умолчанию уровень логирования INFO, чтобы изменить уровень
    логирования нужно запустить тесты с переменной окружения LOG_LEVEL,
    например `export LOG_LEVEL=DEBUG`


3. **PyTest расширения.**
   -  функция `step` одновременно декоратор и менеджер контекста
      Allure-шаг, с добавлением логирования начала и конца шага, а также
      причины падения шага.
   -  функция `attach` добавляет любой контент в шаг
   -  функция `allure_testcase` - декоратор добавляет в отчет Allure
      ссылку на тест-кейс в TestRail
   -  функция `strict_step` - декоратор, строгий шаг, добавляется к шагу
      `@strict_step(False)` и указывает, что если шаг упал, то тест
      будет продолжен и результат теста не зависит от этого шага.


4. **pb_common/utils/support.py** - здесь хранятся часто используемы функции
   в разных проектах.


