from attributedict.collections import AttributeDict


AGREE_SECTIONS = ('masterAccount', 'cardContracts', 'currencyContracts', 'goalContracts',
                  'walletContracts', 'loanContracts', 'depositContracts')


class Agreement(AttributeDict):
    """
    Договор клиента
    """
    def __init__(self, context, agreement, integration_id):
        self._context = context
        self.integration_id = integration_id
        if 'cards' in agreement and agreement['cards']:
            for index, card in enumerate(agreement['cards']):
                agreement['cards'][index] = Card(context, card, integration_id)
        super().__init__(agreement)

    @property
    def display_name(self):
        """
        Название договора на дашборде
        """
        return self.displayName

    @property
    def account_id(self):
        """Номер договора"""
        return self.id

    @property
    def main_amount(self):
        """Главная сумма на дашборде"""
        return self.availableLimitAmount.amount / 100
        # TODO: дописать метод!!!

    def show(self, hidden: bool = False) -> bool:
        """
        Отобразить договор на дашборде
        :param hidden: True - скрыть, False - отобразить
        :return: True - успех, False - ошибка
        """
        r = self._context.contract_service.update_contract(
            path=dict(account_id=self.account_id),
            headers={'Authorization': self.integration_id, 'Content-Type': 'application/json'},
            data='{"hidden": %s}' % str(hidden).lower())
        return r.status_code == 200


class Card(AttributeDict):
    """
    Карта к договору клиента
    """
    def __init__(self, context, card, integration_id):
        self._context = context
        self.integration_id = integration_id
        self._card_no = ''
        super().__init__(card)

    @property
    def card_no(self):
        """
        Номер карты - не маскированный
        """
        if self._card_no != '':
            return self._card_no

        v_cards = self._context.vabank_provider.execute(f'SELECT * FROM CRP_CARDS WHERE accountid={self.contractId}')
        for v_card in v_cards:
            if getattr(self, 'cardNo', None):
                self._card_no = None
            if v_card['CARDNO'][-4:] == self.number[-4:]:
                self._card_no = v_card['CARDNO']
        return self._card_no

    @property
    def quartile(self):
        """
        Третья квартиль карты
        """
        if self.card_no:
            return self.card_no[8:12]

    @property
    def display_name(self):
        """
        Название карты на дашборде
        """
        return f'{self.prefixDisplayName} «{self.displayName}»'


class Agreements(list):
    """
    Список договоров клиента
    """
    def __init__(self, context, dashboard):
        self._context = context
        self._dashboard = dashboard
        self._agreements = []

        for group_agreements in AGREE_SECTIONS:
            agreements = self._dashboard.get(group_agreements)
            if not agreements:
                continue
            elif isinstance(agreements, list):
                agrees = [Agreement(context, agr, self.integration_id) for agr in agreements]
            elif isinstance(agreements, dict):
                agrees = [Agreement(context, agreements, self.integration_id)]
            else:
                raise TypeError('Неожиданный тип договора')

            self._agreements.extend(agrees)
            self._dashboard[group_agreements] = agrees

        super().__init__(self._agreements)

    @property
    def curacc(self):
        """Сбер. счет"""
        current = self._dashboard.get('masterAccount', [])
        if current:
            return current[0]
        return {}

    @property
    def cards(self):
        """Все договора - Карты"""
        return self._dashboard.get('cardContracts', [])

    @property
    def credits(self):
        """Все договора - Кредиты"""
        return self._dashboard.get('loanContracts', [])

    @property
    def deposits(self):
        """Все договора - Вклады"""
        return self._dashboard.get('depositContracts', [])

    @property
    def wallets(self):
        """Все договора - Кошельки"""
        return self._dashboard.get('walletContracts', [])

    @property
    def goals(self):
        """Все договора - Цели"""
        return self._dashboard.get('goalContracts', [])

    @property
    def quartile_card(self):
        """
        Получить любую квартиль карты для регистрации
        :return: квартиль карты
        """
        agrees = [self.curacc] if self.curacc else []
        agrees += self.cards
        for agree in agrees:
            if 'cards' in agree and agree['cards']:
                for card in agree.cards:
                    if card.state == 'active' and card.isPrimary:
                        return card.quartile
        raise AttributeError('Не найдено действующих карт к Сбер. счету и карточным продуктам')

    @property
    def integration_id(self):
        """
        Идентификатор клиента из ответа дашборд сервис
        :return: integration_id
        """
        return self._dashboard.session.user.id
