import ldap
import time

from pb_common.db import BaseDbProvider
from pb_common.utils.logger import get_logger

logger = get_logger(__name__)


class LdapProvider(BaseDbProvider):

    def __init__(self, host, port, name, login, password):
        # параметры аутетификации к БД
        super().__init__()
        self.auth = {"host": host, "port": port, "db": name, "user": login, "password": password}
        self.cursor = None
        self.logger = logger

    @property
    def get_cursor(self):
        if not self.cursor:
            self.connect()
        return self.cursor

    def connect(self):
        """Коннект к LDAP
        :return: курсор
        """
        connect_attempts = 300
        attempts_interval = 0.2
        self.logger.debug("Initialize LDAP connection...")
        while connect_attempts >= 0:
            try:
                ldap_conn = ldap.initialize('ldap://%s:%d' % (self.auth["host"], int(self.auth["port"])))
                ldap_conn.protocol_version = ldap.VERSION3
                self.logger.debug("Binding LDAP connection...")
                ldap_conn.simple_bind(self.auth["user"], self.auth["password"])
                self.logger.debug("Connected to LDAP: {}".format(ldap_conn))
                self.cursor = ldap_conn
                return self
            except ldap.LDAPError as e:
                self.logger.error("Error: {}.\n".format(e))
                if connect_attempts <= 0:
                    raise e
                self.logger.debug("Attempts left: {}".format(connect_attempts - 1))
                self.logger.debug("Re-try to LDAP binding ...")
                time.sleep(attempts_interval)
            finally:
                connect_attempts -= 1

    def execute(self, action, req=None, search_scope=ldap.SCOPE_SUBTREE, retrieve_attributes=None, **kwargs):
        """
        Выполнить запрос.
        :param action:
        :param req:
        :param search_scope:
        :param retrieve_attributes:
        :return:
        """
        execute_attempts = 300
        attempts_interval = 0.2
        action = action.lower()
        while execute_attempts >= 0:
            try:
                if action == "select":
                    return self._search(req=req, scope=search_scope, attrs=retrieve_attributes)
                elif action == "update":
                    return self._update(dn=req['DN'], name=req['name'], value=str(req['value']))
                elif action == "delete":
                    return self._delete(req)
                else:
                    raise AttributeError("Unsupported 'action': {}".format(action))
            except ldap.LDAPError as e:
                self.logger.error(e)
                if execute_attempts <= 0:
                    raise e
                self.close()
                time.sleep(attempts_interval)
                self.logger.debug("Attempts left: {}".format(execute_attempts - 1))
                self.logger.debug("Re-try to LDAP execute ...")
            finally:
                execute_attempts -= 1

    def _delete(self, base_dn):
        self.logger.debug("DN to delete: {}".format(base_dn))
        search = self.get_cursor.search_s(base_dn, ldap.SCOPE_ONELEVEL)
        for dn, _ in search:
            self._delete(dn)
        self.logger.debug("Deleting LDAP row: {}".format(base_dn))
        self.get_cursor.delete_s(base_dn)

    def _search(self, req, scope, attrs):
        self.logger.debug('Search by: {}'.format(req))
        ldap_result = self.get_cursor.search_s(base=self.auth["db"], scope=scope, filterstr=req, attrlist=attrs)
        self.logger.debug("Search LDAP result: %s" % str(ldap_result))
        return ldap_result

    def _update(self, dn, name, value):
        """
        Обновить значение атрибута name объекта DN
        :param dn: объект AD
        :param name: название атрибута
        :param value: значение атрибута
        """
        self.logger.debug("Update {} by name '{}' to value : {}".format(dn, name, value))
        if name != 'cn':
            mod_replace = [(ldap.MOD_REPLACE, name, value.encode())]
            self.get_cursor.modify_s(dn, mod_replace)
            self.logger.debug("Update LDAP replace: {}".format(str(mod_replace)))
            return
        new_dn = name + "=" + value
        self.get_cursor.rename_s(dn, new_dn)
        self.logger.debug("Update LDAP: {}".format(str(new_dn)))

    def close(self):
        """ Закрываем соединение.
        """
        try:
            self.cursor.unbind_s()
        except Exception as e:
            self.logger.error(e)
        finally:
            self.cursor = None
            self.logger.debug("Closing opened LDAP connections.")
