# -*- coding: utf-8 -*-
from cassandra.cluster import Cluster, ExecutionProfile
from cassandra import ConsistencyLevel
from cassandra.auth import PlainTextAuthProvider
from cassandra.policies import WhiteListRoundRobinPolicy
from cassandra.query import SimpleStatement, dict_factory

from pb_common.db import BaseDbProvider
from pb_common.utils import support
from pb_common.utils.logger import get_logger

logger = get_logger(__name__)


def item_to_dict(item):
    if isinstance(item, dict):
        for k, v in list(item.items()):
            try:
                v_dict = item_to_dict(v)
                del item[k]
                item.update({k: v_dict})
            except Exception:
                pass
    elif isinstance(item, list):
        for n, r in enumerate(item):
            try:
                v_dict = item_to_dict(r)
                item.pop(n)
                item.insert(n, v_dict)
            except Exception:
                pass
    elif getattr(item, '_fields', False):
        fields = item._fields
        item = {field.upper(): item_to_dict(getattr(item, field)) for field in fields}
    else:
        try:
            item = item_to_dict(item.__dict__)
        except Exception:
            pass
    return item


class CassandraProvider(BaseDbProvider):
    """
    Класс работы с CassandraDriver.
    """

    def __init__(self, host: list, port: int, name: str, login: str, password: str):
        # параметры аутетификации к БД
        super().__init__()
        self._host = host  # может храниться массив хостов, str->list
        self._port = port
        self._name = name
        self._login = login
        self._password = password

        # параметры для работы внутри БД
        self._cluster = None
        self._session = None

    @property
    def _get_cursor(self):
        if not self._session:
            self.connect()
        return self._session

    def close(self):
        try:
            self._session.shutdown()
            self._cluster.shutdown()
        except Exception as e:
            logger.error(e)
        finally:
            self._session = None
            self._cluster = None
            logger.debug("Disconnect all sessions cassandra.")

    def connect(self):
        logger.debug("Create obj PlainTextAuthProvider for cassandra.")
        auth_provider = PlainTextAuthProvider(username=self._login, password=self._password)
        balance_policy = WhiteListRoundRobinPolicy(hosts=self._host)
        logger.debug("Create cluster for cassandra.")
        self._cluster = Cluster(contact_points=self._host, port=self._port, auth_provider=auth_provider,
                                connect_timeout=60, control_connection_timeout=15,
                                execution_profiles={"user": ExecutionProfile(
                                     consistency_level=ConsistencyLevel.LOCAL_QUORUM,
                                     load_balancing_policy=balance_policy,
                                     row_factory=dict_factory
                                 )})
        logger.debug("Create connect with cassandra.")
        logger.debug("Creating session...")
        self._session = self._cluster.connect(keyspace=self._name)
        logger.debug("Create session for cluster cassandra: {}".format(self._session))
        return self

    def execute(self, req, params=None):
        """ Выполнить запрос.
        :param params: параметры, если используется prepare statement
        :param req: сql-запрос
        :return: результат операции
        """
        support.validate_type(params, None, dict)
        if not params:
            params = {}
        try:
            logger.debug("Execute CQL query: %s" % req)
            query = SimpleStatement(req, consistency_level=ConsistencyLevel.LOCAL_QUORUM)
            result = self._get_cursor.execute(query, params, timeout=30.0)
            if len(result.current_rows) > 0:
                response = [item_to_dict(index) for index in result.current_rows]
            else:
                response = result.current_rows
            logger.debug("Response %s rows from cassandra." % len(response))
            return response
        except Exception as e:
            logger.error("Query execute exception: {}".format(e))
            raise e
