import os
import cx_Oracle
from cx_Oracle import LOB

from pb_common.db import BaseDbProvider
from pb_common.utils import support
from pb_common.utils.logger import get_logger

logger = get_logger(__name__)


def dict_factory(cur):
    """Преобразование ответа БД из кортежа в словарь"""
    if cur.description:
        columns = [d[0].upper() for d in cur.description]
    else:
        columns = []

    def row(*args):
        return dict(zip(columns, [x.read() if isinstance(x, LOB) else x for x in args]))
    return row


class OracleProvider(BaseDbProvider):
    """
    Класс работы с Oracle.
    """
    os.environ['NLS_LANG'] = 'RUSSIAN.CL8MSWIN1251'

    def __init__(self, host: str, port: int, name: str, login: str, password: str):
        # параметры аутетификации к БД
        super().__init__()
        self._connection_str = '%s/%s@%s:%s/%s' % (login, password, host, port, name)
        logger.debug("Connection string: {}".format(self._connection_str))
        self._conn = None
        self._cursor = None

    @property
    def get_cursor(self):
        if not self._cursor or not self._conn:
            self.connect()
        return self._cursor

    def close(self):
        self._cursor_close()
        self._connection_close()
        logger.debug("Close Oracle DB connection.")

    def _cursor_close(self):
        try:
            self._cursor.close()
        except Exception as e:
            logger.error(e)
        finally:
            self._cursor = None

    def _connection_close(self):
        try:
            self._conn.close()
        except Exception as e:
            logger.error(e)
        finally:
            self._conn = None

    def __rows_to_dict_list(self):
        if self.get_cursor.description:
            columns = [i[0] for i in self.get_cursor.description]
            return [
                dict(list(zip(columns, [x.read() if isinstance(x, LOB) else x for x in row])))
                for row in self.get_cursor
            ]
        else:
            return []

    def connect(self):
        if self._conn:
            self.close()
        self._conn = cx_Oracle.connect(self._connection_str)
        self._conn.autocommit = True
        logger.debug("Open Oracle connection.")
        self._cursor = self._conn.cursor()
        logger.debug("Connected to Oracle: {}".format(self._cursor))
        return self

    def execute(self, req, params=None):
        support.validate_type(params, None, dict, list, tuple)
        if not params:
            params = ()
        try:
            logger.debug("Execute SQL query: %s" % req)
            self.get_cursor.execute(req, params)
            response = self.__rows_to_dict_list()
            logger.debug("Count result row: %s" % len(response))
            return response
        except Exception as e:
            logger.error("Query execute exception: {}".format(e))
            raise e

    def fetch_one(self):
        row = self.get_cursor.fetchone()
        if row and self.get_cursor.description:
            columns = [i[0] for i in self.get_cursor.description]
            return dict(list(zip(columns, [x if isinstance(x, str) else x for x in row])))
        else:
            return {}

    def fetch_many(self, limit=1000):
        logger.debug('Take {} rows.'.format(limit))
        rows = self.get_cursor.fetchmany(limit)
        if rows and self.get_cursor.description:
            columns = [i[0] for i in self.get_cursor.description]
            r = [dict(list(zip(columns, [x if isinstance(x, str) else x for x in row]))) for row in rows]
        else:
            r = []
        logger.debug('Received {} rows.'.format(len(r)))
        return r
