import json
import time

from pb_common.client.client import Client
from pb_common.utils.logger import get_logger

__author__ = "Maksim Senchuk"

logger = get_logger(__name__)


class ClientProviderError(Exception):
    pass


class ClientProvider(object):
    """Получение тестовых данных из Бендера, генерация новых данных"""

    def __init__(self, context):
        self._context = context
        self.user = {}
        self._alias = None

    def __call__(self, action='GET', task_id=None, alias=None, is_client=False, timeout=0, **kwargs):
        """
        Общий метод работы с тестовыми данными
        :param action: действие - соответствует вызываемому методу Бендера
        :param task_id: task_id клиента
        :param alias: alias клиента
        :param is_client: признак, используется для создания новых тестовых данных в Бендере, если True - создан клиент
        :param timeout: время в секундах в течение которого, происходит опрос Бендера - подготовлены ли тестовые данные
        :param kwargs: параметры для создания новых тестовых данных
        """
        if action.upper() == 'GET':
            self.user = self._get(task_id, alias)
            return Client(self._context, **self.user['response'])
        elif action.upper() == 'GET_TEST_CLIENT':
            user = self._get(task_id, alias)
            self._release(user['task_id'])
            return Client(self._context, **user['response'])
        elif action.upper() == 'RELEASE':
            return self._release()
        elif action.upper() == 'RERUN':
            return self._rerun()
        elif action.upper() == 'CREATE_TASK':
            self._create_task(**kwargs)
            start_time = time.time()
            while time.time() - start_time <= timeout:
                r = self._get_task()
                if r['state'] == 'COMPLETED':
                    if is_client:
                        self.user = r
                        return Client(self._context, **self.user['response'])
                    return r['response']
                delta = timeout - (time.time() - start_time)
                sleep = int(delta / (int(timeout / 10))) if int(delta / (int(timeout / 10))) >= 1 else 1
                logger.debug('Ожидать следующей проверки статуса задачи: {} сек.'.format(sleep))
                time.sleep(sleep)
            if timeout > 0:
                raise ClientProviderError('Не получен финальный статус подготовки тестовых данных '
                                          'за {} секунд по задаче {}'.format(timeout, self.user['task_id']))
        elif action.upper() == 'GET_TASK':
            return self._get_task(**kwargs)
        elif action.upper() == 'DELETE':
            return self._delete()
        else:
            raise ClientProviderError('Неизвестное действие: {}'.format(action))

    @property
    def alias(self):
        """
        Получить alias тестового клиента
        :return: alias
        """
        return self._alias

    def _get(self, task_id, alias):
        """
        Получить клиента из хранилища тестовых пользователей для автотестов.
        При этом если клиента нельзя использовать параллельно в нескольких тестах - клиенту будет
        установлен признак "занят" и обновится время последнего использования клиента.
        После выполнения теста - клиента нужно освободить методом _release.
        Если клиента можно использовать только один раз - нужно вызвать метод _delete после теста.
        :param task_id: идентификатор задачи в Бендере для получения конкретного клиента
        :param alias: алиас клиента, для получения любого свободного клиента по этому алиасу
        :return: задача
        """
        if task_id:
            r = self._context.bender_hub.acquire_client(
                path=dict(item='task_id'), params=dict(value=task_id, env=self._context.env))
        elif alias:
            r = self._context.bender_hub.acquire_client(
                path=dict(item='alias'), params=dict(value=alias, env=self._context.env))
        else:
            raise AttributeError('Не передан task_id или alias')
        if r.status_code == 200:
            self._alias = alias
            return r.json()
        raise ClientProviderError('Ошибка при поиске клиента: {}'.format(r.text))

    def _release(self, task_id=None):
        """
        Освободить клиента в хранилище пользователей для автотестов.
        Отправляет запрос в Бендер, чтобы снять метку - "занят" с клиента.
        :return: True - успех, False - что-то пошло не так
        """
        if not task_id and self.user:
            task_id = self.user['task_id']
        elif not self.user and not task_id:
            logger.debug("Нет клиента, которого можно освободить")
            return False

        r = self._context.bender_hub.release_client(path=dict(client_uuid=task_id))
        self._alias = None
        logger.debug("Release client: {}".format(r.text))
        return r.status_code == 200

    def _delete(self):
        """
        Пометить клиента удаленным в хранилище пользователей для автотестов.
        :return: True - успех, False - что-то пошло не так
        """
        r = self._context.bender_hub.delete_client(path=dict(client_uuid=self.user['task_id']))
        self._alias = None
        logger.debug("Delete client: {}".format(r.text))
        return r.status_code == 200

    def _rerun(self):
        """
        Запустить на выполнение новую задачу, такую же как имеющаяся
        :return: True - успех, False - что-то пошло не так
        """
        r = self._context.bender_hub.re_create_client(path=dict(client_uuid=self.user['task_id']))
        return r.status_code == 200

    def _create_task(self, **kwargs):
        """
        Запустить на выполнение новую задачу
        """
        kwargs.update(dict(env=self._context.env))
        r = self._context.bender_hub.create_task(data=json.dumps(kwargs))
        if r.status_code == 200:
            self.user.update(r.json())
        else:
            raise ClientProviderError('Не удалось создать новую задачу: {}'.format(r.text))

    def _get_task(self, task_id=None):
        """
        Получить данные по задаче
        :param task_id: идентификатор задачи
        :return: задача
        """
        if not task_id:
            task_id = self.user['task_id']
        r = self._context.bender_hub.get_task(path=dict(client_uuid=task_id))
        if r.status_code == 200:
            return r.json()
        logger.warning(r.text)
        return {}
