from copy import deepcopy
from functools import partial
from urllib.parse import quote_plus

import requests

from pb_common.services import AbstractRequestProvider
from pb_common.utils.logger import get_logger

__author__ = "Maksim Senchuk"


logger = get_logger(__name__)


class Dbo2MethodUndefined(Exception):
    pass


class Dbo2ServiceProvider(AbstractRequestProvider):
    def __init__(self, service: str, protocol: str, host: str, port: int, methods: dict, path=None, headers=None):
        self.service_name = service
        self._protocol = protocol
        self._host = host
        self._port = port
        self._methods = methods
        self._path = path
        self._headers = dict(headers) if headers else {}

    def base_url(self):
        url = "{protocol}://{host}:{port}".format(protocol=self._protocol, host=self._host, port=self._port)
        if self._path:
            url += self._path if self._path.startswith('/') else '/' + self._path
        return url

    def __getattr__(self, item):
        if dict(self._methods).get(item):
            http_method = self._methods[item][0]
            url_path = self._methods[item][1]
            url = self.base_url() + (url_path if url_path.startswith('/') else '/{}'.format(url_path))
            return partial(self.send, method=http_method, url=url)
        else:
            raise Dbo2MethodUndefined("Не найден {} метод {} сервиса в списке разрешенных методов: {}".format(
                item, self.service_name, self._methods))

    def send(self, method, url, path: dict = None, params: dict = None, quote: bool = False,  session=None, **kwargs):
        """
        Отправить запрос
        :param method: метод запроса GET/POST/PUT и т.д
        :param url: урл куда отправлять запрос
        :param path: словарь ключей и значений, где ключи это динамические данные в урл
        :param params: словарь ключ-значение, который добавится в урл в виде ?ключ1=значение1&ключ2=значение2
        :param quote: bool - переводить ли строку поле ? в url-строку
        :param session: сессия запросов
        :param kwargs: остальные параметры запроса
        :return: Response
        """
        if kwargs.get('headers'):
            h = deepcopy(self._headers)
            h.update(kwargs['headers'])
            kwargs['headers'] = h
        else:
            kwargs['headers'] = self._headers
        if not kwargs['headers'].get('User-Agent') or not kwargs['headers'].get('user-agent'):
            kwargs['headers'].update({'User-Agent': 'AutoTest User Agent by: {}'.format(__author__)})
        if path:
            url = url.format(**path)
        if quote and params:
            params = {k: quote_plus(v) for k, v in params.items()}
        logger.info("Send to '{}' request: {} {}".format(self.service_name, method, url))
        if list(filter(lambda x: 'image' in kwargs['headers'][x], kwargs.get('headers', {}).keys())):
            logger.info("Request params: {} ,'data': <image>".format(kwargs['headers']))
        else:
            logger.info("Request params: {}".format(kwargs))
        if kwargs.get('data'):
            if isinstance(kwargs['data'], str):
                kwargs['data'] = kwargs['data'].encode('utf8')
        if not session:
            session = requests.Session()
        r = session.request(method, url, params=params, **kwargs)
        logger.info("Response: {} {}\nHeaders: {}\nBody: {}".format(r.status_code, r.reason, r.headers, r.text))
        return r
